@@ -8,11 +8,22 @@ fn main() {
     println!("cargo:rerun-if-env-changed=CC");
     println!("cargo:rerun-if-env-changed=CFLAGS");
     println!("cargo:rerun-if-env-changed=LDFLAGS");
-    println!("cargo:rerun-if-env-changed=RUSTFLAGS");
+    println!("cargo:rerun-if-env-changed=ORJSON_DISABLE_AVX512");
+    println!("cargo:rerun-if-env-changed=ORJSON_DISABLE_SIMD");
     println!("cargo:rerun-if-env-changed=ORJSON_DISABLE_YYJSON");
+    println!("cargo:rerun-if-env-changed=RUSTFLAGS");
+    println!("cargo:rustc-check-cfg=cfg(intrinsics)");
+    println!("cargo:rustc-check-cfg=cfg(optimize)");
+    println!("cargo:rustc-check-cfg=cfg(Py_3_10)");
+    println!("cargo:rustc-check-cfg=cfg(Py_3_11)");
+    println!("cargo:rustc-check-cfg=cfg(Py_3_12)");
+    println!("cargo:rustc-check-cfg=cfg(Py_3_13)");
+    println!("cargo:rustc-check-cfg=cfg(Py_3_8)");
+    println!("cargo:rustc-check-cfg=cfg(Py_3_9)");
 
-    let py_cfg = pyo3_build_config::get();
-    py_cfg.emit_pyo3_cfgs();
+    for cfg in pyo3_build_config::get().build_script_outputs() {
+        println!("{cfg}");
+    }
 
     if let Some(true) = version_check::supports_feature("core_intrinsics") {
         println!("cargo:rustc-cfg=feature=\"intrinsics\"");
@@ -26,8 +37,11 @@ fn main() {
         println!("cargo:rustc-cfg=feature=\"strict_provenance\"");
     }
 
-    if let Some(true) = version_check::supports_feature("trusted_len") {
-        println!("cargo:rustc-cfg=feature=\"trusted_len\"");
+    #[cfg(any(target_arch = "x86_64", target_arch = "aarch64"))]
+    if env::var("ORJSON_DISABLE_SIMD").is_err() {
+        if let Some(true) = version_check::supports_feature("portable_simd") {
+            println!("cargo:rustc-cfg=feature=\"unstable-simd\"");
+        }
     }
 
     if env::var("ORJSON_DISABLE_YYJSON").is_ok() {
