@@ -3,9 +3,15 @@
 mod cache;
 mod deserializer;
 mod error;
+mod pyobject;
 mod utf8;
 
-pub use cache::KeyMap;
-pub use cache::KEY_MAP;
+#[cfg(not(feature = "yyjson"))]
+mod json;
+
+#[cfg(feature = "yyjson")]
+mod yyjson;
+
+pub use cache::{KeyMap, KEY_MAP};
 pub use deserializer::deserialize;
 pub use error::DeserializeError;
